@@ -32,17 +32,20 @@ from sensiml.datamanager import (
 from sensiml.datamanager.captures import CaptureExistsError
 from sensiml.datamanager.knowledgepack import delete_knowledgepack, get_knowledgepack
 from sensiml.datasets import DataSets
-from sensiml.dclproj.upload import upload_project
+from sensiml.dclproj.upload import upload_project, upload_project_dcli
 
 # from sensiml.base.exceptions import *
 from sensiml.pipeline import Pipeline
 
 config_dir = user_config_dir(__name__.split(".")[0], False)
+
 SERVER_URL = "https://sensiml.cloud/"
 
+AUTH2_PRIMARY = True
+
 logger = logging.getLogger("SensiML")
 
-__version__ = "2022.3.2"
+__version__ = "2023.1.1"
 
 
 def project_set(func):
@@ -126,7 +129,12 @@ class Client(object):
         auth_url = server + "oauth/"
 
         self._connection = Connection(
-            server=server, auth_url=auth_url, path=path, insecure=insecure, **kwargs
+            server=server,
+            auth_url=auth_url,
+            path=path,
+            insecure=insecure,
+            oauth2_primary=AUTH2_PRIMARY,
+            **kwargs,
         )
 
         if not self.validate_client_version(skip_validate):
@@ -160,7 +168,7 @@ class Client(object):
         if skip_validate:
             return True
 
-        url = "version"
+        url = "version/"
 
         response = self._connection.request("get", url)
         response_data, err = utility.check_server_response(response)
@@ -745,3 +753,16 @@ class Client(object):
             return
 
         upload_project(self, name, dclproj_path)
+
+    def upload_project_dcli(self, name: str, dcli_path: str):
+        """Upload a .dcli project to the server
+
+        Args:
+            name (str): name of the project to create
+            dcli_path (str): path to the .dclproj file
+        """
+        if name in self.list_projects()["Name"].values:
+            print("Project with this name already exists.")
+            return
+
+        upload_project_dcli(self, name, dcli_path)
