@@ -2,7 +2,7 @@ try:
     from http.cookiejar import CookiePolicy
 except ImportError:
     from cookielib import CookiePolicy
-
+from datetime import datetime, timezone
 import errno
 import json as JSON
 import logging
@@ -52,21 +52,34 @@ class Warnings:
 class Connection(object):
     """Connection class"""
 
-    def __init__(self, server=None, username=None, password=None, **kwargs):
+    def __init__(
+        self,
+        server=None,
+        username=None,
+        password=None,
+        oauth2_primary=True,
+        api_key=None,
+        debug=False,
+        **kwargs,
+    ):
         """Create Connection object
 
         Args:
+            oauth2_primary (bool, required): oauth2 | api_key
             server (str, optional): Server URL or name in config file.
             username (str, optional)
             password (str, optional)
+            api_key (str, optional)
+            debug (bool, False): adds additional print statements to see url information
             **kwargs (optional): Mostly used when passing `server` as URL. See below.
 
         Keyword Args:
             insecure (bool): Whether or not to require valid SSL/TLS in requests
-            path (str, optional): When specifiying a server name instead of a URL, the path of the config file.
+            path (str, optional): When specifying a server name instead of a URL, the path of the config file.
             server_name (str, optional): user-readable name of server. defaults to the server domain.
             url (str, optional): Alternative name for server argument.
             auth_url (str, optional): URL of the oauth server. Defaults to server url.
+
         """
         # Check url parameter to allow direct passing of ConnectionConfig instances
 
@@ -93,6 +106,7 @@ class Connection(object):
             self.client_secret = csid
             self.username = username
 
+        self.debug = debug
         self.token_url = self.auth_url.rstrip("/") + "/token/"
         self.max_retries = kwargs.pop("max_retries", 3)
 
@@ -120,9 +134,9 @@ class Connection(object):
             os.environ["OAUTHLIB_INSECURE_TRANSPORT"] = ""
             self._verify = True
 
-        self._login(password)
+        self._login(oauth2_primary, password, api_key)
 
-    def _login(self, password, attempt=0):
+    def _build_session(self, attempt=0):
         if attempt >= self.max_retries:
             raise LoginAttemptsExceededError()
 
@@ -134,11 +148,83 @@ class Connection(object):
         self._set_session_retries()
         self.session.cookies.set_policy(BlockAll())
 
+    def _clear_session(self):
+        self._save_token(None)
+        self.logout(self._token_filename)
+
+    def _login(self, oauth2_primary, password=None, api_key=None, attempt=0):
+        if password is not None:
+            self._login_with_oauth2(password, attempt)
+        elif api_key is not None:
+            self._login_with_api_key(api_key, attempt)
+        elif oauth2_primary:
+            self._login_with_oauth2(password, attempt)
+        else:
+            self._login_with_api_key(api_key, attempt)
+
+    def _login_with_api_key(self, api_key=None, isError=False, attempt=0):
+
+        self._build_session(attempt=attempt)
+
+        def request_api_key():
+            if isError:
+                print(
+                    "Invalid API key. Input correct API Key, that you can generate in API Keys -> Generate API Key"
+                )
+            else:
+                print(
+                    "Connect to the cloud using API key, that you can generate in API Keys -> Generate API Key"
+                )
+            return input("Enter API token key:")
+
+        # get token from json file
+        if not api_key and os.path.isfile(self._token_filename):
+            try:
+                with open(self._token_filename, "r") as f:
+                    apiKeyData = JSON.load(f)
+            except Exception as e:
+                self._clear_session()
+            else:
+                if apiKeyData:
+                    api_key = apiKeyData.get("access_token")
+
+        # request token from user
+        if not api_key:
+            api_key = request_api_key()
+
+        # if API key is still empty exit
+        if not api_key:
+            return 0
+
+        data = {
+            "access_token": api_key,
+            "token_type": "Bearer",
+        }
+        self.session.token = data
+
+        try:
+            self._save_token(self.session.token)
+            response = self.request("get", "team-info/")
+        except Exception as e:
+            self._clear_session()
+            self._login_with_api_key(isError=True, api_key=None, attempt=attempt + 1)
+        else:
+            if response.status_code != 200:
+                self._clear_session()
+                self._login_with_api_key(
+                    isError=True, api_key=None, attempt=attempt + 1
+                )
+            else:
+                print("Client connected to the cloud successfully")
+
+    def _login_with_oauth2(self, password, attempt=0):
+
+        self._build_session(attempt=attempt)
+
         if self.username is None or password is None:
             if os.path.exists(self._token_filename):
                 with open(self._token_filename, "r") as f:
                     self.session.token = JSON.load(f)
-
                 try:
                     self.session.refresh_token(
                         self.token_url,
@@ -181,7 +267,7 @@ class Connection(object):
                     os.remove(self._token_filename)
                 password = None
                 self.username = None
-                return self._login(password, attempt=attempt + 1)
+                return self._login_with_oauth2(password, attempt=attempt + 1)
             self._save_token(self.session.token)
 
         logger.debug("Login completed")
@@ -212,7 +298,14 @@ class Connection(object):
         if self._token_last_updated != os.path.getmtime(self._token_filename):
             with open(self._token_filename, "r") as f:
                 self.session.token = JSON.load(f)
+
         while attempt < self.max_retries:
+
+            if self.debug:
+                print(
+                    f"{ datetime.now(timezone.utc)} {method} {url} attempt: {attempt}"
+                )
+
             if paged:
                 return PagedRequest(
                     self.session,
@@ -224,7 +317,7 @@ class Connection(object):
                     verify=verify,
                     client_id=self.client_id,
                     client_secret=self.client_secret,
-                    **kwargs
+                    **kwargs,
                 )
             else:
                 response = self.session.request(
@@ -236,12 +329,18 @@ class Connection(object):
                     verify=verify,
                     client_id=self.client_id,
                     client_secret=self.client_secret,
-                    **kwargs
+                    **kwargs,
                 )
+
+            attempt += 1
+
+            if response.status_code in [504]:
+                continue
+
             # temp workaround for bug in django-oauth-toolkit
             if response.status_code not in [401]:
                 return response
-            attempt += 1
+
             try:
                 # If normal autorefresh fails we force it
                 token = self.session.refresh_token(
