@@ -88,6 +88,15 @@ class ClientPlatformDescriptions:
 
         return platformDescriptions
 
+    def list_processors(self):
+        all_platforms = self.get_platforms()
+        procs = {"name": {}}
+        for plat in all_platforms:
+            for processor in plat.processors:
+                if processor.display_name not in procs["name"].keys():
+                    procs["name"][processor.display_name] = processor.uuid
+        return DataFrame(procs)
+
     def __call__(self):
         return self.__str__()
 
