@@ -15,6 +15,7 @@ class FeatureFile(Base):
         "last_modified",
         "is_features",
         "label_column",
+        "number_rows",
     ]
 
     def __init__(
@@ -26,6 +27,7 @@ class FeatureFile(Base):
         is_features=True,
         uuid=None,
         label_column="",
+        number_rows=None,
     ):
         self._connection = connection
         self._project = project
@@ -35,6 +37,7 @@ class FeatureFile(Base):
         self.name = name
         self.path = path
         self.label_column = label_column
+        self.number_rows = number_rows
 
     # Maintain campatibility with old filename attr
     @property
@@ -150,3 +153,57 @@ class FeatureFile(Base):
         response = self._connection.request("get", url)
 
         return response
+
+    def compute_analysis(self, analysis_type="UMAP", **kwargs):
+        """Calls the REST API to compute the analysis for the feature file.
+
+        Args:
+            analysis_type (str): the type of clustering analysis, ie "UMAP" (default), "TSNE" and "PCA".
+
+        Kwargs:
+            shuffle_seed (int): random seed to shuffle and resample feature vector
+            analysis_seed (int): random state of the analysis (default is 0)
+            n_neighbor (int): The size of local neighborhood (in terms of number of neighboring sample points) used for manifold approximation. If not specified, default is the number of unique labels.
+            n_components (int): The dimension of the output result. Default is 2. 'n_components' is adjusted based on the method, dimension of the feature vector and number of samples
+            n_sample (int): Maximum number of output samples. Default is 1000.
+
+        Returns:
+            A JSON response containing the metadata of the generated analysis.
+
+        Example:
+            >>> feature_file = client.get_featurefile(<feature-file uuid>)
+            >>> response = feature_file.compute_analysis(analysis_type="PCA", shuffle_seed=13, n_components=5)
+            >>> response.json()
+
+        """
+
+        url = "project/{0}/featurefile-analysis/{1}/".format(
+            self._project.uuid, self.uuid
+        )
+
+        json_payload = dict({"analysis_type": analysis_type}, **kwargs)
+
+        response = self._connection.request("post", url, json=json_payload)
+
+        return response
+
+    def list_analysis(self):
+        """Calls the REST API and retrieve list of computed analysis for featurefile
+
+        Returns:
+            JSON response holding the list of all computed analysis
+        """
+
+        url = "project/{0}/featurefile-analysis/{1}/".format(
+            self._project.uuid, self.uuid
+        )
+
+        response = self._connection.request("get", url)
+
+        response_data, err = utility.check_server_response(response)
+
+        if err is True:
+            print(err)
+            return response
+
+        return response_data
