@@ -139,8 +139,9 @@ class FeatureFiles:
             list (featurefiles)
         """
         err = False
-        url = "project/{0}/featurefile/{1}".format(self._project.uuid, uuid)
+        url = "project/{0}/featurefile/{1}/".format(self._project.uuid, uuid)
         response = self._connection.request("get", url)
+
         try:
             response_data, err = utility.check_server_response(response)
         except ValueError:
