@@ -1,10 +1,10 @@
 import json
-from numpy import float64, int64
-import sensiml.base.utility as utility
 import logging
 import time
-from sensiml.datamanager.base import Base, BaseSet
 
+import sensiml.base.utility as utility
+from numpy import float64, int64
+from sensiml.datamanager.base import Base, BaseSet
 
 logger = logging.getLogger(__name__)
 
@@ -23,6 +23,7 @@ class Label(Base):
         "name",
         "value_type",
         "is_dropdown",
+        "metadata",
         "last_modified",
         "created_at",
     ]
@@ -82,6 +83,10 @@ class Label(Base):
     def _metadata(self):
         return False
 
+    @property
+    def metadata(self):
+        return False
+
     @property
     def is_dropdown(self):
         return self._is_dropdown
