@@ -87,7 +87,7 @@ class Projects:
             project
         """
         project = Project(self._connection)
-        if len(dict) is not 0:
+        if len(dict) != 0:
             project.initialize_from_dict(dict)
         return project
 
