@@ -91,7 +91,7 @@ class Sandboxes:
             sandbox object
         """
         sandbox = Sandbox(self._connection, self._project)
-        if len(dict) is not 0:
+        if len(dict) != 0:
             sandbox.initialize_from_dict(dict)
         return sandbox
 
