@@ -8,8 +8,6 @@ class Team(object):
 
     _uuid = ""
     _name = None
-    _cpu_clock_time = None
-    _cpu_clock_time_history = None
 
     def __init__(self, connection):
         self._connection = connection
