@@ -1,10 +1,9 @@
-from sensiml.dclproj.dclproj import DCLProject
 from sensiml.dclproj.csv_to_dcli import to_dcli
-from sensiml.dclproj.datasegments import (
-    segment_list_to_datasegments,
+from sensiml.dclproj.datasegmentsv2 import (
     audacity_to_datasegments,
+    segment_list_to_datasegments,
 )
-
+from sensiml.dclproj.dclproj import DCLProject
 
 __all__ = [
     "DCLProject",
