@@ -1,19 +1,19 @@
-from sensiml.method_calls.featurefilecall import FeatureFileCall
-from sensiml.method_calls.capturefilecall import CaptureFileCall
-from sensiml.method_calls.querycall import QueryCall
-from sensiml.method_calls.functioncall import FunctionCall
-from sensiml.method_calls.trainandvalidationcall import TrainAndValidationCall
 from sensiml.method_calls.augmentationcall import AugmentationCall
 from sensiml.method_calls.augmentationcallset import AugmentationCallSet
+from sensiml.method_calls.capturefilecall import CaptureFileCall
+from sensiml.method_calls.classifiercall import ClassifierCall
+from sensiml.method_calls.datafilecall import DataFileCall
+from sensiml.method_calls.featurefilecall import FeatureFileCall
+from sensiml.method_calls.functioncall import FunctionCall
 from sensiml.method_calls.generatorcall import GeneratorCall
 from sensiml.method_calls.generatorcallset import GeneratorCallSet
+from sensiml.method_calls.optimizercall import OptimizerCall
+from sensiml.method_calls.querycall import QueryCall
 from sensiml.method_calls.selectorcall import SelectorCall
 from sensiml.method_calls.selectorcallset import SelectorCallSet
-from sensiml.method_calls.validationmethodcall import ValidationMethodCall
-from sensiml.method_calls.classifiercall import ClassifierCall
-from sensiml.method_calls.optimizercall import OptimizerCall
+from sensiml.method_calls.trainandvalidationcall import TrainAndValidationCall
 from sensiml.method_calls.trainingalgorithmcall import TrainingAlgorithmCall
-
+from sensiml.method_calls.validationmethodcall import ValidationMethodCall
 
 __all__ = [
     "FunctionCall",
@@ -31,4 +31,5 @@ __all__ = [
     "OptimizerCall",
     "TrainingAlgorithmCall",
     "CaptureFileCall",
+    "DataFileCall",
 ]
