@@ -382,7 +382,7 @@ class Pipeline(object):
         lock=True,
         silent=True,
         renderer=None,
-        version=1,
+        version=2,
     ):
         """
         Execute automated pipeline asynchronously.
@@ -515,24 +515,18 @@ class Pipeline(object):
         Where A, B and C are the parameters to search over. Additionally, for each step you may want to search over more than
         one of a functions configurable parameters. To do this simply add another element to the functions dictionary.
 
-            grid_params = {"Name Of Function":{"Name of Parameter 1":[ A, B, C],
-                                            "Name of Parameter 2":[ D, E]}}
+            grid_params = {"Name Of Function":{"Name of Parameter 1":[ A, B, C], "Name of Parameter 2":[ D, E]}}
 
         This will tell grid search to search over 6 different parameter spaces.
 
         You can also specify more than one step to search over in grid params. This is done by adding another element to
         the function level of the grid_params dictionary.
 
-            grid_params = {"Name Of Function":{"Name of Parameter 1":[ A, B, C],
-                                            "Name of Parameter 2":[ D, E]},
-                                            "Name of Function 2":{"Name of Parameter":[1, 2, 3, 4, 5, 6]}}
+            grid_params = {"Name Of Function":{"Name of Parameter 1":[ A, B, C], "Name of Parameter 2":[ D, E]}, "Name of Function 2":{"Name of Parameter":[1, 2, 3, 4, 5, 6]}}
 
         Example:
 
-            grid_params = {'Windowing':{"window_size": [100,200],'delta':[100]},
-                        'selector_set': {"Recursive Feature Elimination":{'number_of_features':[10, 20]}},
-                        'Hierarchical Clustering with Neuron Optimization': {'number_of_neurons':[10,20]}
-                        }
+            grid_params = {'Windowing':{"window_size": [100,200],'delta':[100]}, 'selector_set': {"Recursive Feature Elimination":{'number_of_features':[10, 20]}}, 'Hierarchical Clustering with Neuron Optimization': {'number_of_neurons':[10,20]}}
 
             results, stats = client.pipeline.grid_search(grid_params)
 
@@ -1110,13 +1104,14 @@ class Pipeline(object):
             n_columns (int): The number of columns to return as a combination.
             columns (list): if None, will use the columns in the input feature generator, otherwise will use this list provided
 
-        fg =  {'name':'Downsample with Min Max Scaling', 'params':{"columns": ['gyrZ','gyrX'] , "new_length": 15}}
+        Example
+            >>> fg =  {'name':'Downsample with Min Max Scaling', 'params':{"columns": ['gyrZ','gyrX'] , "new_length": 15}}
+            >>> fg_new = client.pipeline.iterate_columns(fg, n_columns=1)
 
-        fg_new = client.pipeline.iterate_columns(fg, n_columns=1)
+            >>> print(fg_new)
 
-        >> print(fg_new)
-        >> [{'name': 'Downsample with Min Max Scaling', 'params': {'columns': ['gyrX'], 'new_length': 15}},
-        {'name': 'Downsample with Min Max Scaling', 'params': {'columns': ['gyrX'], 'new_length': 15}}]
+            Output:
+            [{'name': 'Downsample with Min Max Scaling', 'params': {'columns': ['gyrX'], 'new_length': 15}}, {'name': 'Downsample with Min Max Scaling', 'params': {'columns': ['gyrX'], 'new_length': 15}}]
         """
 
         if n_columns is None:
@@ -1355,6 +1350,7 @@ class Pipeline(object):
         dtype=np.int32,
         int8_input_type=True,
         shape=None,
+        verbose=True,
     ):
         """Converts a SensiML feature vector Dataframe into a train, test, validate set of Numpy arrays to use in training a tensorflow model
 
@@ -1419,16 +1415,17 @@ class Pipeline(object):
             y_values_encoded, [TRAIN_SPLIT, TEST_SPLIT]
         )
 
-        print("-----  Summary  -----")
-        print("Class Map:" + str(class_map))
-        print("Train:\n total: ", x_train.shape[0])
-        print(" by class:", y_train.sum(axis=0))
+        if verbose:
+            print("-----  Summary  -----")
+            print("Class Map:" + str(class_map))
+            print("Train:\n total: ", x_train.shape[0])
+            print(" by class:", y_train.sum(axis=0))
 
-        print("Validate:\n total: ", x_validate.shape[0])
-        print(" by class:", y_validate.sum(axis=0))
+            print("Validate:\n total: ", x_validate.shape[0])
+            print(" by class:", y_validate.sum(axis=0))
 
-        print("Test:\n total: ", x_test.shape[0])
-        print(" by class:", y_test.sum(axis=0))
+            print("Test:\n total: ", x_test.shape[0])
+            print(" by class:", y_test.sum(axis=0))
 
         return (
             x_train.astype(dtype),
