@@ -6,6 +6,13 @@ import zipfile
 from ctypes import CDLL
 from tkinter import E
 
+try:
+    from typing import Literal
+except:
+    from typing_extensions import Literal
+
+from typing import Dict, List, Optional
+
 from numpy import ndarray
 from pandas import DataFrame, Series
 
@@ -79,22 +86,12 @@ class struct_pme_model_header(ctypes.Structure):
     ]
 
 
-class struct_tf_micro_model_result(ctypes.Structure):
+class struct_model_result(ctypes.Structure):
     __slots__ = ["num_outputs", "output_tensor"]
 
     _fields_ = [
         ("num_outputs", ctypes.c_ubyte),
-        ("output_tensor", ctypes.POINTER(ctypes.c_float)),
-    ]
-
-
-class struct_decision_tree_model_result(ctypes.Structure):
-    __slots__ = ["number_of_trees", "classification_counts", "class_percentages"]
-
-    _fields_ = [
-        ("number_of_trees", ctypes.c_ubyte),
-        ("classification_counts", ctypes.POINTER(ctypes.c_uint8)),
-        ("class_percentages", ctypes.POINTER(ctypes.c_float)),
+        ("output_tensor", ctypes.POINTER(ctypes.c_int16)),
     ]
 
 
@@ -158,15 +155,21 @@ class Model(object):
 
 class SMLRunner(object):
     """
-    This class provides a python interface to the knowledgpack libary generated by SensiML.
-    After downloading the knowledgpack for your platform, you can instantiate this function
-    by passing the path to libsensiml.so
+    This class provides a python interface to the Knowledge Pack library generated by SensiML.
 
-        sml = SMLRunner(<path_to_libsensiml.so>)
+
+    Args:
+        path (str): Path to the downloaded Knowledge Pack .zip, the libsensiml.so, or libsensiml.dll
+        class_map (Optional[Dict], optional): The class map to use for this model. Defaults to None.
+
+    Example:
+
+    After downloading the Knowledge Pack for Windows or Linux.
+
+        sml = SMLRunner(<path_to_knowledgepack.zip>)
         sml.init_model()
 
-    Then, you can pythonically call all of the c functions in kb.h passing data like you
-    normally would.  For example, if you have a dataframe df which has your sensor data
+    Then, you can use the python API to call the C functions in kb.h. For example, if you have a Dataframe df which has your sensor data
     stored in it,
 
         print(df.head(2))
@@ -174,18 +177,20 @@ class SMLRunner(object):
             9    10     11
             12   8      23
 
-    you can call run_model and pass a single sample at a time to run the model as you would
-    on the device.
+    you can call recognize_capture API
 
-        for index in range(len(df)):
-            category = sml.run_model(df.iloc[index].values, 0, model_index)
-            if category >= 0:
-                print(category)
-                sml.reset_model(0)
+    results = recognize_capture(df)
 
     """
 
-    def __init__(self, path, class_map=None):
+    def __init__(self, path: str, class_map: Optional[Dict] = None):
+        """Initialize the SMLRunner with a downloaded Knowledge Pack
+
+        Args:
+            path (str): Path to the downloaded Knowledge Pack .zip, the libsensiml.so, or libsensiml.dll
+            class_map (Optional[Dict], optional): The class map to use for this model. Defaults to None.
+
+        """
         self._run_type = None
         self._model_initialized = False
 
@@ -346,7 +351,7 @@ class SMLRunner(object):
             ctypes.c_int,
             ctypes.c_void_p,
         ]
-        self._kb_get_classification_result_info.restype = None
+        self._kb_get_classification_result_info.restype = ctypes.c_int
 
         self._add_custom_pattern_to_model = clf_lib.kb_add_custom_pattern_to_model
         self._add_custom_pattern_to_model.argtypes = [
@@ -459,7 +464,7 @@ class SMLRunner(object):
     def init_model(self):
         """
         This will initialize the parameters of all the models, it should be run once before
-        running anything else.
+        running any other APIs.
         """
         if not self._model_initialized:
             self._model_initialized = True
@@ -467,7 +472,7 @@ class SMLRunner(object):
 
     def flush_model_buffer(self, model_index):
         """
-        This will flush the data buffer of a knowledge pack.
+        This will flush the data buffer of a Knowledge Pack.
         """
         if not self._initialized():
             return
@@ -478,8 +483,8 @@ class SMLRunner(object):
 
     def add_custom_pattern_to_model(self, model_index, feature_vector, category, aif):
         """
-        updates the model by adding the a new pattern to the database
-        with a new category and influence field
+        Updates a PME model by adding the last pattern to the internal database
+        with the specified category and influence field
 
         Args:
            model_index(int): Index of the model to update
@@ -511,7 +516,7 @@ class SMLRunner(object):
 
     def add_last_pattern_to_model(self, model_index, category, aif):
         """
-        Updates the model by adding the a the last feature vector created to the database
+        Updates a PME model by adding the last feature vector created to the database
         with a new category and influence field
 
         Args:
@@ -565,17 +570,17 @@ class SMLRunner(object):
 
         return res
 
-    def retrain_model(self, model_index):
+    def retrain_model(self, model_index: int = 0):
         """
-        After a model has been scored, this will restraing the model according
-        to the models supported reinforcement learning approach
+        After a model has been scored, this will retrain the model according
+        to the models online learning approach
 
         Args:
            model_index(int): Index of the model to update
 
         Returns:
            int: 0 if model does not support dynamic scoring
-                1 if model was succesfully scored
+                1 if model was scored
         """
         if not self._initialized():
             return
@@ -590,7 +595,13 @@ class SMLRunner(object):
         return ret
 
     def run_model(
-        self, data_sample, model_index, debug_log=True, model_api="run_model"
+        self,
+        data_sample,
+        model_index: int = 0,
+        debug_log: bool = True,
+        model_api: Literal[
+            "run_model", "run_model_cascade_features", "sml_recognition_run"
+        ] = "run_model",
     ):
         """
         This is the main entry point into the pipeline. It takes a single timepoint of data as an array. Adds that sample to the internal ring buffer
@@ -601,7 +612,7 @@ class SMLRunner(object):
             data_sample(array):  single timepoint of data as an array from all sensors
             nsensors(int): unused
             model_index(int): Index of the model to run
-            debug_log(bool): Prints the debug output from the knowledge pack
+            debug_log(bool): Prints the debug output from the Knowledge Pack
             model_api(str): use run_model, run_model_cascade_features or sml_recognition_run. These correspond to the API's in the kb.h file
 
          Returns:
@@ -645,12 +656,28 @@ class SMLRunner(object):
 
     def recognize_capture(
         self,
-        data,
-        model_index=0,
-        model_api="run_model",
-        get_feature_vector=False,
-        get_sensor_data=False,
+        data: DataFrame,
+        model_index: int = 0,
+        model_api: Literal[
+            "run_model", "run_model_cascade_features", "sml_recognition_run"
+        ] = "run_model",
+        get_feature_vector: bool = False,
+        get_sensor_data: bool = False,
+        get_output_tensor: bool = True,
     ):
+        """_summary_
+
+        Args:
+            data (DataFrame): Input sensor data
+            model_index (int, optional): The index of model to use. Defaults to 0.
+            model_api (Literal[&quot;run_model&quot;,&quot;run_model_cascade_features&quot;,&quot;sml_recognition_run&quot;], optional): Model API to use. Defaults to "run_model".
+            get_feature_vector (bool, optional): Includes the feature vector output in the returned result. Defaults to False.
+            get_sensor_data (bool, optional): Includes the sensor data in the returned result. Defaults to False.
+            get_output_tensor (bool, optional): Includes the output tensor probabilities in the returned result (if applicable). Defaults to True.
+
+        Returns:
+            _type_: _description_
+        """
 
         M = []
 
@@ -687,6 +714,9 @@ class SMLRunner(object):
                         sensor_data = self.get_segment_data(model_index)
                         output_columns["sensor_data_%d" % i] = sensor_data[i]
 
+                if get_output_tensor:
+                    output_columns["output_tensor"] = self.get_model_result(model_index)
+
                 M.append(output_columns)
 
                 # After a classification is received, call sml_reset model to advance the internal buffer.
@@ -700,7 +730,9 @@ class SMLRunner(object):
 
         return DataFrame(M)
 
-    def run_segment(self, data_segment, model_index, debug_log=False):
+    def run_segment(
+        self, data_segment: DataFrame, model_index: int = 0, debug_log: bool = False
+    ):
         """
         Add a segment of data to the model as input. Then runs the model on the current segment, skipping the data streaming steps.
 
@@ -709,7 +741,7 @@ class SMLRunner(object):
         Args:
             data(array):  Array of timseries data
             model_index(int): Index of the model to run
-            debug_log(bool): Prints the debug output from the knowledge pack
+            debug_log(bool): Prints the debug output from the Knowledge Pack
 
          Returns:
              Classification results will be 0 if unknown through the classification numbers
@@ -754,7 +786,7 @@ class SMLRunner(object):
 
         return ret
 
-    def reset_model(self, model_index):
+    def reset_model(self, model_index: int = 0):
         """
         Advances the model state to so that it is ready for a more input data. Use
         only after classification steps.
@@ -773,7 +805,7 @@ class SMLRunner(object):
 
         self._reset_model(model_index_ctype)
 
-    def flush_model(self, model_index):
+    def flush_model(self, model_index: int = 0):
         """
         Deletes all of the patterns in the database.
 
@@ -788,22 +820,9 @@ class SMLRunner(object):
 
         self._flush_model(model_index_ctype)
 
-    def get_model_map(self):
+    def get_model(self, model_index: int = 0):
         """
-        Print the model name to map relationship
-        """
-        print("not currently supported.")
-
-        return
-
-        with pipes() as (out, _):
-            self._print_model_map()
-
-        return json.loads(out.read())
-
-    def get_model(self, model_index):
-        """
-        Prints the model weights and info for current model if supported
+        Prints the model weights and info for current model if supported by this model type.
 
         Args:
             model index - number of axes in the data stream
@@ -847,14 +866,30 @@ class SMLRunner(object):
 
         return DataFrame(model_scores)[["ID", "ERR"] + cats].style.apply(_color, axis=1)
 
-    def get_class_map(self, model_index):
+    def get_class_map(self, model_index: int = 0) -> Dict:
+        """Returns the class map for the model
+
+        Args:
+            model_index (int, optional): The model index. Defaults to 0.
+
+        Returns:
+            dict: Class map for the specified model_index
+        """
 
         if not self._initialized():
             return None
 
         return self.class_map[model_index]
 
-    def get_model_header(self, model_index):
+    def get_model_header(self, model_index: int = 0):
+        """Returns the model header for a PME model
+
+        Args:
+            model_index (int, optional): The model index. Defaults to 0.
+
+        Returns:
+            struct_pme_model_header: A struct object containing information about the PME model header
+        """
 
         if not self._initialized():
             return
@@ -867,7 +902,16 @@ class SMLRunner(object):
 
         return pme_model_header
 
-    def get_model_pattern(self, model_index, pattern_index):
+    def get_model_pattern(self, model_index: int = 0, pattern_index: int = 0):
+        """Returns the pattern for a PME model at a particular index
+
+        Args:
+            model_index (int, optional): The model index. Defaults to 0.
+            pattern_index (int, optional): The index of the pattern in the database to retrieve. Defaults to 0.
+
+        Returns:
+            struct_pme_pattern: struct_pme_pattern object
+        """
 
         if not self._initialized():
             return
@@ -881,7 +925,15 @@ class SMLRunner(object):
 
         return pme_pattern
 
-    def get_segment_data(self, model_index: int):
+    def get_segment_data(self, model_index: int = 0) -> List:
+        """Returns the segment data currently stored in the models internal ring buffer.
+
+        Args:
+            model_index (int, optional): The model index. Defaults to 0.
+
+        Returns:
+            list: the sensor data stored in the model ring buffer
+        """
 
         if not self._initialized():
             return
@@ -910,7 +962,15 @@ class SMLRunner(object):
 
         return sensor_data
 
-    def get_model_framelen(self, model_index: int):
+    def get_model_framelen(self, model_index: int = 0) -> int:
+        """Returns the number of channels expected by the model
+
+        Args:
+            model_index (int): The model index
+
+        Returns:
+            int: The number of input channels used by the model
+        """
 
         if not self._initialized():
             return
@@ -919,9 +979,20 @@ class SMLRunner(object):
 
         framelen = self._kb_get_model_framelen(model_index_ctype)
 
-        return framelen
+        return int(framelen)
 
-    def get_feature_vector(self, model_index, feature_vector_buffer_size=4096):
+    def get_feature_vector(
+        self, model_index: int = 0, feature_vector_buffer_size: int = 4096
+    ) -> List:
+        """Get the current feature vector from the model
+
+        Args:
+            model_index (int, optional): The model index. Defaults to 0.
+            feature_vector_buffer_size (int, optional): The size of the array to use for getting the feature vector (This should be equal to or greater the size of the feature vector used by the model). Defaults to 4096.
+
+        Returns:
+            List: The models current feature vector
+        """
 
         if not self._initialized():
             return
@@ -936,7 +1007,15 @@ class SMLRunner(object):
 
         return list(feature_vector_ctype[: feature_length_ctype.value])
 
-    def get_segment_length(self, model_index):
+    def get_segment_length(self, model_index: int = 0) -> int:
+        """Get the length of the current segment
+
+        Args:
+            model_index (int, optional): The model index. Defaults to 0.
+
+        Returns:
+            int: The length of the current segment
+        """
 
         if not self._initialized():
             return
@@ -948,7 +1027,15 @@ class SMLRunner(object):
 
         return segment_length.value
 
-    def get_feature_bank_number(self, model_index):
+    def get_feature_bank_number(self, model_index: int = 0) -> int:
+        """Get the number of feature banks this model uses
+
+        Args:
+            model_index (int, optional): The model index. Defaults to 0.
+
+        Returns:
+            int: The number of feature banks
+        """
 
         if not self._initialized():
             return
@@ -962,9 +1049,12 @@ class SMLRunner(object):
 
         return feature_bank_number.value
 
-    def set_feature_vector(self, model_index, feature_vector):
-        """
-        set a model buffers feature vector
+    def set_feature_vector(self, model_index: int, feature_vector: list):
+        """Sets the models feature vector to the specified value
+
+        Args:
+            model_index (int): The model index
+            feature_vector (list): A list containing the features to set the current models internal feature vector too
         """
 
         if not self._initialized():
@@ -978,9 +1068,14 @@ class SMLRunner(object):
 
         return self._set_feature_vector(model_index_ctype, feature_vector_array)
 
-    def recognize_feature_vector(self, model_index):
-        """
-        classify the feature vector that is currently in the model buffer
+    def recognize_feature_vector(self, model_index: int = 0) -> int:
+        """Classify the feature vector that is currently in the model buffer
+
+        Args:
+            model_index (int, optional): The model index. Defaults to 0.
+
+        Returns:
+            int: the classification result
         """
 
         if not self._initialized():
@@ -990,14 +1085,13 @@ class SMLRunner(object):
 
         return self._recognize_feature_vector(model_index_ctype)
 
-    def get_model_result(self, model_index: int, model_type: str):
+    def get_model_result(self, model_index: int = 0, tensor_size: int = 128):
         """
-        Get the model results object for the most recent classification of a model type
-
+        Get the model tensor output for the most recent classification
 
         Args:
             model_index (int): the model index to use
-            model_type (str): tensorflow lite for microcontrollers or decision tree ensemble
+            tensor_size (int): the size of the tensor, should be larger than the largest output_tensor size
 
         Returns:
             model results object for the specified model
@@ -1005,28 +1099,26 @@ class SMLRunner(object):
         if not self._initialized():
             return
 
-        if model_type.lower() == "tensorflow lite for microcontrollers":
-            model_index_ctype = ctypes.c_int(model_index)
-            model_result = struct_tf_micro_model_result()
-            output_tensor = (ctypes.c_float * 128)()
-            model_result.output_tensor = ctypes.POINTER(output_tensor)
-        elif model_type.lower() == "decision tree ensemble":
-            model_index_ctype = ctypes.c_int(model_index)
-            model_result = struct_decision_tree_model_result()
-            classification_counts = (ctypes.c_uint8 * 128)()
-            class_percentages = (ctypes.c_float * 128)()
-            model_result.classification_counts = ctypes.POINTER(classification_counts)
-            model_result.class_percentages = ctypes.POINTER(class_percentages)
-        else:
-            raise Exception("Model Type Not Currently Supported!")
+        model_index_ctype = ctypes.c_int(model_index)
+        model_result = struct_model_result()
+        output_tensor = (ctypes.c_int16 * tensor_size)()
+        model_result.output_tensor = output_tensor
 
-        self._classification_result_info(model_index_ctype, model_result)
+        self._kb_get_classification_result_info(
+            model_index_ctype, ctypes.cast(ctypes.byref(model_result), ctypes.c_void_p)
+        )
 
-        return model_result
+        outputs = []
+        for i in range(model_result.num_outputs):
+            outputs.append(model_result.output_tensor[i])
 
-    def knowledgepack(self, model_index, distance_mode=0, feature_list=None):
+        return outputs
+
+    def knowledgepack(
+        self, model_index: int = 0, distance_mode: int = 0, feature_list=None
+    ):
         """
-        Turn into a knowledgepack Model Object (Only works for PME)
+        Turn into a Knowledge Pack model object (only works for PME)
         """
         self.init_model()
         neuron_array = self.get_model(model_index).to_dict("records")
@@ -1041,8 +1133,7 @@ class SMLRunner(object):
 
     def run_and_score_model(self, data_sample, category, model_index=0):
         """
-        Will Run ans core a model based on passing a single sample at a time.
-
+        Runs a PME model and updates the internal model score.
         """
 
         ret = self.run_model(data_sample, model_index)
@@ -1064,7 +1155,7 @@ class SMLRunner(object):
 
     def run_and_score_model_on_segment(self, segment, category, model_index=0):
         """
-        will run and score a model based on a segment of data.
+        Run a PME model and updates the internal scores based on a segment of data.
 
         (Warning: using this call will flush the models internal ring buffer. Alternating between this and run_and_score model may produce incorrect results or crashes)
 
